@@ -63,7 +63,7 @@
 
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/session.h"
-#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/torrent.h"
 #include "base/exceptions.h"
 #include "base/iconprovider.h"
 #include "base/logger.h"
@@ -81,15 +81,16 @@
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
 #include "base/utils/string.h"
+#include "base/version.h"
 #include "applicationinstancemanager.h"
 #include "filelogger.h"
 
 #ifndef DISABLE_GUI
-#include "addnewtorrentdialog.h"
+#include "gui/addnewtorrentdialog.h"
 #include "gui/uithememanager.h"
 #include "gui/utils.h"
-#include "mainwindow.h"
-#include "shutdownconfirmdialog.h"
+#include "gui/mainwindow.h"
+#include "gui/shutdownconfirmdialog.h"
 #endif // DISABLE_GUI
 
 #ifndef DISABLE_WEBUI
@@ -181,12 +182,14 @@ Application::Application(int &argc, char **argv)
         m_fileLogger = new FileLogger(fileLoggerPath(), isFileLoggerBackup(), fileLoggerMaxSize(), isFileLoggerDeleteOld(), fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
 
     Logger::instance()->addMessage(tr("qBittorrent %1 started", "qBittorrent v3.2.0alpha started").arg(QBT_VERSION));
-    if (portableModeEnabled) {
+    if (portableModeEnabled)
+    {
         Logger::instance()->addMessage(tr("Running in portable mode. Auto detected profile folder at: %1").arg(profileDir));
         if (m_commandLineArgs.relativeFastresumePaths)
             Logger::instance()->addMessage(tr("Redundant command line flag detected: \"%1\". Portable mode implies relative fastresume.").arg("--relative-fastresume"), Log::WARNING); // to avoid translating the `--relative-fastresume` string
     }
-    else {
+    else
+    {
         Logger::instance()->addMessage(tr("Using config directory: %1").arg(Profile::instance()->location(SpecialFolder::Config)));
     }
 }
@@ -212,7 +215,7 @@ const QBtCommandLineParameters &Application::commandLineArgs() const
 
 bool Application::isFileLoggerEnabled() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_ENABLED, true).toBool();
+    return settings()->loadValue(KEY_FILELOGGER_ENABLED, true);
 }
 
 void Application::setFileLoggerEnabled(const bool value)
@@ -226,8 +229,8 @@ void Application::setFileLoggerEnabled(const bool value)
 
 QString Application::fileLoggerPath() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_PATH,
-            {specialFolderLocation(SpecialFolder::Data) + LOG_FOLDER}).toString();
+    return settings()->loadValue(KEY_FILELOGGER_PATH
+        , QString {specialFolderLocation(SpecialFolder::Data) + LOG_FOLDER});
 }
 
 void Application::setFileLoggerPath(const QString &path)
@@ -239,7 +242,7 @@ void Application::setFileLoggerPath(const QString &path)
 
 bool Application::isFileLoggerBackup() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_BACKUP, true).toBool();
+    return settings()->loadValue(KEY_FILELOGGER_BACKUP, true);
 }
 
 void Application::setFileLoggerBackup(const bool value)
@@ -251,7 +254,7 @@ void Application::setFileLoggerBackup(const bool value)
 
 bool Application::isFileLoggerDeleteOld() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_DELETEOLD, true).toBool();
+    return settings()->loadValue(KEY_FILELOGGER_DELETEOLD, true);
 }
 
 void Application::setFileLoggerDeleteOld(const bool value)
@@ -263,7 +266,7 @@ void Application::setFileLoggerDeleteOld(const bool value)
 
 int Application::fileLoggerMaxSize() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_MAXSIZEBYTES, DEFAULT_FILELOG_SIZE).toInt();
+    const int val = settings()->loadValue(KEY_FILELOGGER_MAXSIZEBYTES, DEFAULT_FILELOG_SIZE);
     return std::min(std::max(val, MIN_FILELOG_SIZE), MAX_FILELOG_SIZE);
 }
 
@@ -277,7 +280,7 @@ void Application::setFileLoggerMaxSize(const int bytes)
 
 int Application::fileLoggerAge() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_AGE, 1).toInt();
+    const int val = settings()->loadValue(KEY_FILELOGGER_AGE, 1);
     return std::min(std::max(val, 1), 365);
 }
 
@@ -288,7 +291,7 @@ void Application::setFileLoggerAge(const int value)
 
 int Application::fileLoggerAgeType() const
 {
-    const int val = settings()->loadValue(KEY_FILELOGGER_AGETYPE, 1).toInt();
+    const int val = settings()->loadValue(KEY_FILELOGGER_AGETYPE, 1);
     return ((val < 0) || (val > 2)) ? 1 : val;
 }
 
@@ -308,7 +311,7 @@ void Application::processMessage(const QString &message)
         m_paramsQueue.append(params);
 }
 
-void Application::runExternalProgram(const BitTorrent::TorrentHandle *torrent) const
+void Application::runExternalProgram(const BitTorrent::Torrent *torrent) const
 {
     QString program = Preferences::instance()->getAutoRunProgram().trimmed();
     program.replace("%N", torrent->name());
@@ -361,11 +364,13 @@ void Application::runExternalProgram(const BitTorrent::TorrentHandle *torrent) c
     proc.setArguments(argList);
     proc.setCreateProcessArgumentsModifier([](QProcess::CreateProcessArguments *args)
     {
-        if (Preferences::instance()->isAutoRunConsoleEnabled()) {
+        if (Preferences::instance()->isAutoRunConsoleEnabled())
+        {
             args->flags |= CREATE_NEW_CONSOLE;
             args->flags &= ~(CREATE_NO_WINDOW | DETACHED_PROCESS);
         }
-        else {
+        else
+        {
             args->flags |= CREATE_NO_WINDOW;
             args->flags &= ~(CREATE_NEW_CONSOLE | DETACHED_PROCESS);
         }
@@ -387,11 +392,20 @@ void Application::runExternalProgram(const BitTorrent::TorrentHandle *torrent) c
     // enable command injection via torrent name and other arguments
     // (especially when some automated download mechanism has been setup).
     // See: https://github.com/qbittorrent/qBittorrent/issues/10925
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
+    QStringList args = QProcess::splitCommand(program);
+    if (args.isEmpty())
+        return;
+
+    const QString command = args.takeFirst();
+    QProcess::startDetached(command, args);
+#else
     QProcess::startDetached(program);
 #endif
+#endif
 }
 
-void Application::sendNotificationEmail(const BitTorrent::TorrentHandle *torrent)
+void Application::sendNotificationEmail(const BitTorrent::Torrent *torrent)
 {
     // Prepare mail content
     const QString content = tr("Torrent name: %1").arg(torrent->name()) + '\n'
@@ -410,7 +424,7 @@ void Application::sendNotificationEmail(const BitTorrent::TorrentHandle *torrent
                      content);
 }
 
-void Application::torrentFinished(BitTorrent::TorrentHandle *const torrent)
+void Application::torrentFinished(BitTorrent::Torrent *const torrent)
 {
     Preferences *const pref = Preferences::instance();
 
@@ -419,7 +433,8 @@ void Application::torrentFinished(BitTorrent::TorrentHandle *const torrent)
         runExternalProgram(torrent);
 
     // Mail notification
-    if (pref->isMailNotificationEnabled()) {
+    if (pref->isMailNotificationEnabled())
+    {
         Logger::instance()->addMessage(tr("Torrent: %1, sending mail notification").arg(torrent->name()));
         sendNotificationEmail(torrent);
     }
@@ -446,16 +461,19 @@ void Application::allTorrentsFinished()
 
 #ifndef DISABLE_GUI
     // ask confirm
-    if ((action == ShutdownDialogAction::Exit) && (pref->dontConfirmAutoExit())) {
+    if ((action == ShutdownDialogAction::Exit) && (pref->dontConfirmAutoExit()))
+    {
         // do nothing & skip confirm
     }
-    else {
+    else
+    {
         if (!ShutdownConfirmDialog::askForConfirmation(m_window, action)) return;
     }
 #endif // DISABLE_GUI
 
     // Actually shut down
-    if (action != ShutdownDialogAction::Exit) {
+    if (action != ShutdownDialogAction::Exit)
+    {
         qDebug("Preparing for auto-shutdown because all downloads are complete!");
         // Disabling it for next time
         pref->setShutdownWhenDownloadsComplete(false);
@@ -481,51 +499,60 @@ bool Application::sendParams(const QStringList &params)
 void Application::processParams(const QStringList &params)
 {
 #ifndef DISABLE_GUI
-    if (params.isEmpty()) {
+    if (params.isEmpty())
+    {
         m_window->activate(); // show UI
         return;
     }
 #endif
     BitTorrent::AddTorrentParams torrentParams;
-    TriStateBool skipTorrentDialog;
+    std::optional<bool> skipTorrentDialog;
 
-    for (QString param : params) {
+    for (QString param : params)
+    {
         param = param.trimmed();
 
         // Process strings indicating options specified by the user.
 
-        if (param.startsWith(QLatin1String("@savePath="))) {
+        if (param.startsWith(QLatin1String("@savePath=")))
+        {
             torrentParams.savePath = param.mid(10);
             continue;
         }
 
-        if (param.startsWith(QLatin1String("@addPaused="))) {
-            torrentParams.addPaused = param.midRef(11).toInt() ? TriStateBool::True : TriStateBool::False;
+        if (param.startsWith(QLatin1String("@addPaused=")))
+        {
+            torrentParams.addPaused = (param.midRef(11).toInt() != 0);
             continue;
         }
 
-        if (param == QLatin1String("@skipChecking")) {
+        if (param == QLatin1String("@skipChecking"))
+        {
             torrentParams.skipChecking = true;
             continue;
         }
 
-        if (param.startsWith(QLatin1String("@category="))) {
+        if (param.startsWith(QLatin1String("@category=")))
+        {
             torrentParams.category = param.mid(10);
             continue;
         }
 
-        if (param == QLatin1String("@sequential")) {
+        if (param == QLatin1String("@sequential"))
+        {
             torrentParams.sequential = true;
             continue;
         }
 
-        if (param == QLatin1String("@firstLastPiecePriority")) {
+        if (param == QLatin1String("@firstLastPiecePriority"))
+        {
             torrentParams.firstLastPiecePriority = true;
             continue;
         }
 
-        if (param.startsWith(QLatin1String("@skipDialog="))) {
-            skipTorrentDialog = param.midRef(12).toInt() ? TriStateBool::True : TriStateBool::False;
+        if (param.startsWith(QLatin1String("@skipDialog=")))
+        {
+            skipTorrentDialog = (param.midRef(12).toInt() != 0);
             continue;
         }
 
@@ -535,9 +562,7 @@ void Application::processParams(const QStringList &params)
         // be shown and skipTorrentDialog is undefined. The other is when
         // skipTorrentDialog is false, meaning that the application setting
         // should be overridden.
-        const bool showDialogForThisTorrent =
-            ((AddNewTorrentDialog::isEnabled() && skipTorrentDialog == TriStateBool::Undefined)
-             || skipTorrentDialog == TriStateBool::False);
+        const bool showDialogForThisTorrent = !skipTorrentDialog.value_or(!AddNewTorrentDialog::isEnabled());
         if (showDialogForThisTorrent)
             AddNewTorrentDialog::show(param, torrentParams, m_window);
         else
@@ -552,14 +577,13 @@ int Application::exec(const QStringList &params)
     Net::DownloadManager::initInstance();
     IconProvider::initInstance();
 
-    try {
+    try
+    {
         BitTorrent::Session::initInstance();
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::torrentFinished, this, &Application::torrentFinished);
         connect(BitTorrent::Session::instance(), &BitTorrent::Session::allTorrentsFinished, this, &Application::allTorrentsFinished, Qt::QueuedConnection);
 
-#ifndef DISABLE_COUNTRIES_RESOLUTION
         Net::GeoIPManager::initInstance();
-#endif
         ScanFoldersModel::initInstance();
 
 #ifndef DISABLE_WEBUI
@@ -574,7 +598,8 @@ int Application::exec(const QStringList &params)
         new RSS::Session; // create RSS::Session singleton
         new RSS::AutoDownloader; // create RSS::AutoDownloader singleton
     }
-    catch (const RuntimeError &err) {
+    catch (const RuntimeError &err)
+    {
 #ifdef DISABLE_GUI
         fprintf(stderr, "%s", err.what());
 #else
@@ -598,7 +623,8 @@ int Application::exec(const QStringList &params)
             .arg(QString("http://localhost:") + QString::number(pref->getWebUiPort())) + '\n';
     printf("%s", qUtf8Printable(mesg));
 
-    if (pref->getWebUIPassword() == "ARQ77eY1NUZaQsuDHbIMCA==:0WMRkYTUWVT9wVvdDtHAjU9b3b7uB8NR1Gur2hmQCvCDpm39Q+PsJRJPaCU51dEiz+dTzh8qbPsL8WkFljQYFQ==") {
+    if (pref->getWebUIPassword() == "ARQ77eY1NUZaQsuDHbIMCA==:0WMRkYTUWVT9wVvdDtHAjU9b3b7uB8NR1Gur2hmQCvCDpm39Q+PsJRJPaCU51dEiz+dTzh8qbPsL8WkFljQYFQ==")
+    {
         const QString warning = tr("The Web UI administrator username is: %1").arg(pref->getWebUiUsername()) + '\n'
             + tr("The Web UI administrator password is still the default one: %1").arg("adminadmin") + '\n'
             + tr("This is a security risk, please consider changing your password from program preferences.") + '\n';
@@ -608,7 +634,6 @@ int Application::exec(const QStringList &params)
 #else
     UIThemeManager::initInstance();
     m_window = new MainWindow;
-    UIThemeManager::instance()->applyStyleSheet();
 #endif // DISABLE_GUI
 
     m_running = true;
@@ -617,7 +642,8 @@ int Application::exec(const QStringList &params)
     BitTorrent::Session::instance()->startUpTorrents();
 
     m_paramsQueue = params + m_paramsQueue;
-    if (!m_paramsQueue.isEmpty()) {
+    if (!m_paramsQueue.isEmpty())
+    {
         processParams(m_paramsQueue);
         m_paramsQueue.clear();
     }
@@ -633,7 +659,8 @@ bool Application::isRunning()
 #ifdef Q_OS_MACOS
 bool Application::event(QEvent *ev)
 {
-    if (ev->type() == QEvent::FileOpen) {
+    if (ev->type() == QEvent::FileOpen)
+    {
         QString path = static_cast<QFileOpenEvent *>(ev)->file();
         if (path.isEmpty())
             // Get the url instead
@@ -645,7 +672,8 @@ bool Application::event(QEvent *ev)
             m_paramsQueue.append(path);
         return true;
     }
-    else {
+    else
+    {
         return BaseApplication::event(ev);
     }
 }
@@ -673,11 +701,13 @@ void Application::initializeTranslation()
     installTranslator(&m_translator);
 
 #ifndef DISABLE_GUI
-    if (localeStr.startsWith("ar") || localeStr.startsWith("he")) {
+    if (localeStr.startsWith("ar") || localeStr.startsWith("he"))
+    {
         qDebug("Right to Left mode");
         setLayoutDirection(Qt::RightToLeft);
     }
-    else {
+    else
+    {
         setLayoutDirection(Qt::LeftToRight);
     }
 #endif
@@ -719,7 +749,8 @@ void Application::cleanup()
         return;
 
 #ifndef DISABLE_GUI
-    if (m_window) {
+    if (m_window)
+    {
         // Hide the window and don't leave it on screen as
         // unresponsive. Also for Windows take the WinId
         // after it's hidden, because hide() may cause a
@@ -751,9 +782,7 @@ void Application::cleanup()
 
     ScanFoldersModel::freeInstance();
     BitTorrent::Session::freeInstance();
-#ifndef DISABLE_COUNTRIES_RESOLUTION
     Net::GeoIPManager::freeInstance();
-#endif
     Net::DownloadManager::freeInstance();
     Net::ProxyConfigurationManager::freeInstance();
     Preferences::freeInstance();
@@ -765,7 +794,8 @@ void Application::cleanup()
     Utils::Fs::removeDirRecursive(Utils::Fs::tempPath());
 
 #ifndef DISABLE_GUI
-    if (m_window) {
+    if (m_window)
+    {
 #ifdef Q_OS_WIN
         ::ShutdownBlockReasonDestroy(reinterpret_cast<HWND>(m_window->effectiveWinId()));
 #endif // Q_OS_WIN
@@ -776,7 +806,8 @@ void Application::cleanup()
 
     Profile::freeInstance();
 
-    if (m_shutdownAct != ShutdownDialogAction::Exit) {
+    if (m_shutdownAct != ShutdownDialogAction::Exit)
+    {
         qDebug() << "Sending computer shutdown/suspend/hibernate signal...";
         Utils::Misc::shutdownComputer(m_shutdownAct);
     }
