@@ -32,9 +32,8 @@
 
 #include <libtorrent/bencode.hpp>
 #include <libtorrent/create_torrent.hpp>
-#include <libtorrent/storage.hpp>
+#include <libtorrent/file_storage.hpp>
 #include <libtorrent/torrent_info.hpp>
-#include <libtorrent/version.hpp>
 
 #include <QDirIterator>
 #include <QFile>
@@ -46,24 +45,33 @@
 #include "base/utils/fs.h"
 #include "base/utils/io.h"
 #include "base/utils/string.h"
-#include "private/ltunderlyingtype.h"
+#include "base/version.h"
+#include "ltunderlyingtype.h"
 
 namespace
 {
-#if (LIBTORRENT_VERSION_NUM < 10200)
-    using LTCreateFlags = int;
-    using LTPieceIndex = int;
-#else
-    using LTCreateFlags = lt::create_flags_t;
-    using LTPieceIndex = lt::piece_index_t;
-#endif
-
     // do not include files and folders whose
     // name starts with a .
     bool fileFilter(const std::string &f)
     {
         return !Utils::Fs::fileName(QString::fromStdString(f)).startsWith('.');
     }
+
+#if (LIBTORRENT_VERSION_NUM >= 20000)
+    lt::create_flags_t toNativeTorrentFormatFlag(const BitTorrent::TorrentFormat torrentFormat)
+    {
+        switch (torrentFormat)
+        {
+        case BitTorrent::TorrentFormat::V1:
+            return lt::create_torrent::v1_only;
+        case BitTorrent::TorrentFormat::Hybrid:
+            return {};
+        case BitTorrent::TorrentFormat::V2:
+            return lt::create_torrent::v2_only;
+        }
+        return {};
+    }
+#endif
 }
 
 using namespace BitTorrent;
@@ -96,20 +104,24 @@ void TorrentCreatorThread::run()
 
     emit updateProgress(0);
 
-    try {
+    try
+    {
         const QString parentPath = Utils::Fs::branchPath(m_params.inputPath) + '/';
 
         // Adding files to the torrent
         lt::file_storage fs;
-        if (QFileInfo(m_params.inputPath).isFile()) {
+        if (QFileInfo(m_params.inputPath).isFile())
+        {
             lt::add_files(fs, Utils::Fs::toNativePath(m_params.inputPath).toStdString(), fileFilter);
         }
-        else {
+        else
+        {
             // need to sort the file names by natural sort order
             QStringList dirs = {m_params.inputPath};
 
             QDirIterator dirIter(m_params.inputPath, (QDir::AllDirs | QDir::NoDotAndDotDot), QDirIterator::Subdirectories);
-            while (dirIter.hasNext()) {
+            while (dirIter.hasNext())
+            {
                 dirIter.next();
                 dirs += dirIter.filePath();
             }
@@ -118,11 +130,13 @@ void TorrentCreatorThread::run()
             QStringList fileNames;
             QHash<QString, qint64> fileSizeMap;
 
-            for (const auto &dir : asConst(dirs)) {
+            for (const auto &dir : asConst(dirs))
+            {
                 QStringList tmpNames;  // natural sort files within each dir
 
                 QDirIterator fileIter(dir, QDir::Files);
-                while (fileIter.hasNext()) {
+                while (fileIter.hasNext())
+                {
                     fileIter.next();
 
                     const QString relFilePath = fileIter.filePath().mid(parentPath.length());
@@ -140,18 +154,24 @@ void TorrentCreatorThread::run()
 
         if (isInterruptionRequested()) return;
 
+#if (LIBTORRENT_VERSION_NUM >= 20000)
+        lt::create_torrent newTorrent {fs, m_params.pieceSize, toNativeTorrentFormatFlag(m_params.torrentFormat)};
+#else
         lt::create_torrent newTorrent(fs, m_params.pieceSize, m_params.paddedFileSizeLimit
-            , (m_params.isAlignmentOptimized ? lt::create_torrent::optimize_alignment : LTCreateFlags {}));
+            , (m_params.isAlignmentOptimized ? lt::create_torrent::optimize_alignment : lt::create_flags_t {}));
+#endif
 
         // Add url seeds
-        for (QString seed : asConst(m_params.urlSeeds)) {
+        for (QString seed : asConst(m_params.urlSeeds))
+        {
             seed = seed.trimmed();
             if (!seed.isEmpty())
                 newTorrent.add_url_seed(seed.toStdString());
         }
 
         int tier = 0;
-        for (const QString &tracker : asConst(m_params.trackers)) {
+        for (const QString &tracker : asConst(m_params.trackers))
+        {
             if (tracker.isEmpty())
                 ++tier;
             else
@@ -162,9 +182,9 @@ void TorrentCreatorThread::run()
 
         // calculate the hash for all pieces
         lt::set_piece_hashes(newTorrent, Utils::Fs::toNativePath(parentPath).toStdString()
-            , [this, &newTorrent](const LTPieceIndex n)
+            , [this, &newTorrent](const lt::piece_index_t n)
         {
-            sendProgressSignal(static_cast<LTUnderlyingType<LTPieceIndex>>(n), newTorrent.num_pieces());
+            sendProgressSignal(static_cast<LTUnderlyingType<lt::piece_index_t>>(n), newTorrent.num_pieces());
         });
         // Set qBittorrent as creator and add user comment to
         // torrent_info structure
@@ -185,16 +205,20 @@ void TorrentCreatorThread::run()
 
         // create the torrent
         QFile outfile {m_params.savePath};
-        if (!outfile.open(QIODevice::WriteOnly)) {
-            throw RuntimeError {tr("Create new torrent file failed. Reason: %1")
+        if (!outfile.open(QIODevice::WriteOnly))
+        {
+            throw RuntimeError
+            {tr("Create new torrent file failed. Reason: %1")
                 .arg(outfile.errorString())};
         }
 
         if (isInterruptionRequested()) return;
 
         lt::bencode(Utils::IO::FileDeviceOutputIterator {outfile}, entry);
-        if (outfile.error() != QFileDevice::NoError) {
-            throw RuntimeError {tr("Create new torrent file failed. Reason: %1")
+        if (outfile.error() != QFileDevice::NoError)
+        {
+            throw RuntimeError
+            {tr("Create new torrent file failed. Reason: %1")
                 .arg(outfile.errorString())};
         }
         outfile.close();
@@ -202,12 +226,17 @@ void TorrentCreatorThread::run()
         emit updateProgress(100);
         emit creationSuccess(m_params.savePath, parentPath);
     }
-    catch (const std::exception &e) {
+    catch (const std::exception &e)
+    {
         emit creationFailure(e.what());
     }
 }
 
+#if (LIBTORRENT_VERSION_NUM >= 20000)
+int TorrentCreatorThread::calculateTotalPieces(const QString &inputPath, const int pieceSize, const TorrentFormat torrentFormat)
+#else
 int TorrentCreatorThread::calculateTotalPieces(const QString &inputPath, const int pieceSize, const bool isAlignmentOptimized, const int paddedFileSizeLimit)
+#endif
 {
     if (inputPath.isEmpty())
         return 0;
@@ -215,6 +244,10 @@ int TorrentCreatorThread::calculateTotalPieces(const QString &inputPath, const i
     lt::file_storage fs;
     lt::add_files(fs, Utils::Fs::toNativePath(inputPath).toStdString(), fileFilter);
 
+#if (LIBTORRENT_VERSION_NUM >= 20000)
+    return lt::create_torrent {fs, pieceSize, toNativeTorrentFormatFlag(torrentFormat)}.num_pieces();
+#else
     return lt::create_torrent(fs, pieceSize, paddedFileSizeLimit
-        , (isAlignmentOptimized ? lt::create_torrent::optimize_alignment : LTCreateFlags {})).num_pieces();
+        , (isAlignmentOptimized ? lt::create_torrent::optimize_alignment : lt::create_flags_t {})).num_pieces();
+#endif
 }
