@@ -42,7 +42,7 @@
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/magneturi.h"
 #include "base/bittorrent/session.h"
-#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/torrent.h"
 #include "base/exceptions.h"
 #include "base/global.h"
 #include "base/net/downloadmanager.h"
@@ -107,12 +107,7 @@ AddNewTorrentDialog::AddNewTorrentDialog(const BitTorrent::AddTorrentParams &inP
 
     const auto *session = BitTorrent::Session::instance();
 
-    if (m_torrentParams.addPaused == TriStateBool::True)
-        m_ui->startTorrentCheckBox->setChecked(false);
-    else if (m_torrentParams.addPaused == TriStateBool::False)
-        m_ui->startTorrentCheckBox->setChecked(true);
-    else
-        m_ui->startTorrentCheckBox->setChecked(!session->isAddTorrentPaused());
+    m_ui->startTorrentCheckBox->setChecked(!m_torrentParams.addPaused.value_or(session->isAddTorrentPaused()));
 
     m_ui->comboTTM->blockSignals(true); // the TreeView size isn't correct if the slot does it job at this point
     m_ui->comboTTM->setCurrentIndex(!session->isAutoTMMDisabledByDefault());
@@ -120,11 +115,11 @@ AddNewTorrentDialog::AddNewTorrentDialog(const BitTorrent::AddTorrentParams &inP
     populateSavePathComboBox();
     connect(m_ui->savePath, &FileSystemPathEdit::selectedPathChanged, this, &AddNewTorrentDialog::onSavePathChanged);
 
-    const bool rememberLastSavePath = settings()->loadValue(KEY_REMEMBERLASTSAVEPATH, false).toBool();
+    const bool rememberLastSavePath = settings()->loadValue(KEY_REMEMBERLASTSAVEPATH, false);
     m_ui->checkBoxRememberLastSavePath->setChecked(rememberLastSavePath);
 
     m_ui->contentLayoutComboBox->setCurrentIndex(
-                static_cast<int>(m_torrentParams.contentLayout ? *m_torrentParams.contentLayout : session->torrentContentLayout()));
+                static_cast<int>(m_torrentParams.contentLayout.value_or(session->torrentContentLayout())));
 
     m_ui->sequentialCheckBox->setChecked(m_torrentParams.sequential);
     m_ui->firstLastCheckBox->setChecked(m_torrentParams.firstLastPiecePriority);
@@ -135,7 +130,7 @@ AddNewTorrentDialog::AddNewTorrentDialog(const BitTorrent::AddTorrentParams &inP
     // Load categories
     QStringList categories = session->categories().keys();
     std::sort(categories.begin(), categories.end(), Utils::String::naturalLessThan<Qt::CaseInsensitive>);
-    QString defaultCategory = settings()->loadValue(KEY_DEFAULTCATEGORY).toString();
+    auto defaultCategory = settings()->loadValue<QString>(KEY_DEFAULTCATEGORY);
 
     if (!m_torrentParams.category.isEmpty())
         m_ui->categoryComboBox->addItem(m_torrentParams.category);
@@ -170,7 +165,7 @@ AddNewTorrentDialog::~AddNewTorrentDialog()
 
 bool AddNewTorrentDialog::isEnabled()
 {
-    return SettingsStorage::instance()->loadValue(KEY_ENABLED, true).toBool();
+    return SettingsStorage::instance()->loadValue(KEY_ENABLED, true);
 }
 
 void AddNewTorrentDialog::setEnabled(bool value)
@@ -180,7 +175,7 @@ void AddNewTorrentDialog::setEnabled(bool value)
 
 bool AddNewTorrentDialog::isTopLevel()
 {
-    return SettingsStorage::instance()->loadValue(KEY_TOPLEVEL, true).toBool();
+    return SettingsStorage::instance()->loadValue(KEY_TOPLEVEL, true);
 }
 
 void AddNewTorrentDialog::setTopLevel(bool value)
@@ -191,7 +186,7 @@ void AddNewTorrentDialog::setTopLevel(bool value)
 int AddNewTorrentDialog::savePathHistoryLength()
 {
     const int defaultHistoryLength = 8;
-    const int value = settings()->loadValue(KEY_SAVEPATHHISTORYLENGTH, defaultHistoryLength).toInt();
+    const int value = settings()->loadValue(KEY_SAVEPATHHISTORYLENGTH, defaultHistoryLength);
     return qBound(minPathHistoryLength, value, maxPathHistoryLength);
 }
 
@@ -204,14 +199,14 @@ void AddNewTorrentDialog::setSavePathHistoryLength(int value)
 
     settings()->storeValue(KEY_SAVEPATHHISTORYLENGTH, clampedValue);
     settings()->storeValue(KEY_SAVEPATHHISTORY
-        , QStringList(settings()->loadValue(KEY_SAVEPATHHISTORY).toStringList().mid(0, clampedValue)));
+        , QStringList(settings()->loadValue<QStringList>(KEY_SAVEPATHHISTORY).mid(0, clampedValue)));
 }
 
 void AddNewTorrentDialog::loadState()
 {
     Utils::Gui::resize(this, m_storeDialogSize);
     m_ui->splitter->restoreState(m_storeSplitterState);
-    m_headerState = settings()->loadValue(KEY_TREEHEADERSTATE).toByteArray();
+    m_headerState = settings()->loadValue<QByteArray>(KEY_TREEHEADERSTATE);
 }
 
 void AddNewTorrentDialog::saveState()
@@ -280,7 +275,7 @@ bool AddNewTorrentDialog::loadTorrentImpl()
     // Prevent showing the dialog if download is already present
     if (BitTorrent::Session::instance()->isKnownTorrent(infoHash))
     {
-        BitTorrent::TorrentHandle *const torrent = BitTorrent::Session::instance()->findTorrent(infoHash);
+        BitTorrent::Torrent *const torrent = BitTorrent::Session::instance()->findTorrent(infoHash);
         if (torrent)
         {
             if (torrent->isPrivate() || m_torrentInfo.isPrivate())
@@ -321,7 +316,7 @@ bool AddNewTorrentDialog::loadMagnet(const BitTorrent::MagnetUri &magnetUri)
     // Prevent showing the dialog if download is already present
     if (BitTorrent::Session::instance()->isKnownTorrent(infoHash))
     {
-        BitTorrent::TorrentHandle *const torrent = BitTorrent::Session::instance()->findTorrent(infoHash);
+        BitTorrent::Torrent *const torrent = BitTorrent::Session::instance()->findTorrent(infoHash);
         if (torrent)
         {
             if (torrent->isPrivate())
@@ -371,7 +366,7 @@ void AddNewTorrentDialog::showEvent(QShowEvent *event)
 void AddNewTorrentDialog::saveSavePathHistory() const
 {
     // Get current history
-    QStringList history = settings()->loadValue(KEY_SAVEPATHHISTORY).toStringList();
+    auto history = settings()->loadValue<QStringList>(KEY_SAVEPATHHISTORY);
     QVector<QDir> historyDirs;
     for (const QString &path : asConst(history))
         historyDirs << QDir {path};
@@ -489,11 +484,11 @@ void AddNewTorrentDialog::populateSavePathComboBox()
     m_ui->savePath->clear();
 
     // Load save path history
-    const QStringList savePathHistory {settings()->loadValue(KEY_SAVEPATHHISTORY).toStringList()};
+    const auto savePathHistory {settings()->loadValue<QStringList>(KEY_SAVEPATHHISTORY)};
     for (const QString &savePath : savePathHistory)
         m_ui->savePath->addItem(savePath);
 
-    const bool rememberLastSavePath {settings()->loadValue(KEY_REMEMBERLASTSAVEPATH, false).toBool()};
+    const bool rememberLastSavePath {settings()->loadValue(KEY_REMEMBERLASTSAVEPATH, false)};
     const QString defSavePath {BitTorrent::Session::instance()->defaultSavePath()};
 
     if (!m_torrentParams.savePath.isEmpty())
@@ -573,7 +568,7 @@ void AddNewTorrentDialog::accept()
     if (m_contentModel)
         m_torrentParams.filePriorities = m_contentModel->model()->getFilePriorities();
 
-    m_torrentParams.addPaused = TriStateBool(!m_ui->startTorrentCheckBox->isChecked());
+    m_torrentParams.addPaused = !m_ui->startTorrentCheckBox->isChecked();
     m_torrentParams.contentLayout = static_cast<BitTorrent::TorrentContentLayout>(m_ui->contentLayoutComboBox->currentIndex());
 
     m_torrentParams.sequential = m_ui->sequentialCheckBox->isChecked();
@@ -582,13 +577,13 @@ void AddNewTorrentDialog::accept()
     QString savePath = m_ui->savePath->selectedPath();
     if (m_ui->comboTTM->currentIndex() != 1)
     { // 0 is Manual mode and 1 is Automatic mode. Handle all non 1 values as manual mode.
-        m_torrentParams.useAutoTMM = TriStateBool::False;
+        m_torrentParams.useAutoTMM = false;
         m_torrentParams.savePath = savePath;
         saveSavePathHistory();
     }
     else
     {
-        m_torrentParams.useAutoTMM = TriStateBool::True;
+        m_torrentParams.useAutoTMM = true;
     }
 
     setEnabled(!m_ui->checkBoxNeverShow->isChecked());
@@ -660,7 +655,7 @@ void AddNewTorrentDialog::setupTreeview()
 
         // Set torrent information
         m_ui->labelCommentData->setText(Utils::Misc::parseHtmlLinks(m_torrentInfo.comment().toHtmlEscaped()));
-        m_ui->labelDateData->setText(!m_torrentInfo.creationDate().isNull() ? m_torrentInfo.creationDate().toString(Qt::DefaultLocaleShortDate) : tr("Not available"));
+        m_ui->labelDateData->setText(!m_torrentInfo.creationDate().isNull() ? QLocale().toString(m_torrentInfo.creationDate(), QLocale::ShortFormat) : tr("Not available"));
 
         // Prepare content tree
         m_contentModel = new TorrentContentFilterModel(this);
