@@ -68,6 +68,8 @@
 #include "uithememanager.h"
 #include "utils.h"
 
+#define SETTINGS_KEY(name) "OptionsDialog/" name
+
 namespace
 {
     QStringList translatedWeekdayNames()
@@ -88,6 +90,7 @@ namespace
         {
         case QLocale::Arabic: return QString::fromUtf8(C_LOCALE_ARABIC);
         case QLocale::Armenian: return QString::fromUtf8(C_LOCALE_ARMENIAN);
+        case QLocale::Azerbaijani: return QString::fromUtf8(C_LOCALE_AZERBAIJANI);
         case QLocale::Basque: return QString::fromUtf8(C_LOCALE_BASQUE);
         case QLocale::Bulgarian: return QString::fromUtf8(C_LOCALE_BULGARIAN);
         case QLocale::Byelorussian: return QString::fromUtf8(C_LOCALE_BYELORUSSIAN);
@@ -110,6 +113,7 @@ namespace
             case QLocale::UnitedKingdom: return QString::fromUtf8(C_LOCALE_ENGLISH_UNITEDKINGDOM);
             default: return QString::fromUtf8(C_LOCALE_ENGLISH);
             }
+        case QLocale::Estonian: return QString::fromUtf8(C_LOCALE_ESTONIAN);
         case QLocale::Finnish: return QString::fromUtf8(C_LOCALE_FINNISH);
         case QLocale::French: return QString::fromUtf8(C_LOCALE_FRENCH);
         case QLocale::Galician: return QString::fromUtf8(C_LOCALE_GALICIAN);
@@ -127,7 +131,7 @@ namespace
         case QLocale::Latvian: return QString::fromUtf8(C_LOCALE_LATVIAN);
         case QLocale::Lithuanian: return QString::fromUtf8(C_LOCALE_LITHUANIAN);
         case QLocale::Malay: return QString::fromUtf8(C_LOCALE_MALAY);
-        case QLocale::Norwegian: return QString::fromUtf8(C_LOCALE_NORWEGIAN);
+        case QLocale::NorwegianBokmal: return QString::fromUtf8(C_LOCALE_NORWEGIAN);
         case QLocale::Occitan: return QString::fromUtf8(C_LOCALE_OCCITAN);
         case QLocale::Polish: return QString::fromUtf8(C_LOCALE_POLISH);
         case QLocale::Portuguese:
@@ -167,9 +171,10 @@ private:
 
 // Constructor
 OptionsDialog::OptionsDialog(QWidget *parent)
-    : QDialog(parent)
-    , m_refreshingIpFilter(false)
-    , m_ui(new Ui::OptionsDialog)
+    : QDialog {parent}
+    , m_ui {new Ui::OptionsDialog}
+    , m_storeDialogSize {SETTINGS_KEY("Size")}
+    , m_storeHSplitterSize {SETTINGS_KEY("HorizontalSplitterSizes")}
 {
     qDebug("-> Constructing Options");
     m_ui->setupUi(this);
@@ -563,7 +568,7 @@ OptionsDialog::OptionsDialog(QWidget *parent)
     for (QSpinBox *widget : asConst(findChildren<QSpinBox *>()))
         widget->installEventFilter(wheelEventEater);
 
-    loadWindowState();
+    Utils::Gui::resize(this, m_storeDialogSize);
     show();
     // Have to be called after show(), because splitter width needed
     loadSplitterState();
@@ -604,7 +609,13 @@ OptionsDialog::~OptionsDialog()
 {
     qDebug("-> destructing Options");
 
-    saveWindowState();
+    // save dialog states
+    m_storeDialogSize = size();
+
+    QStringList hSplitterSizes;
+    for (const int size : asConst(m_ui->hsplitter->sizes()))
+        hSplitterSizes.append(QString::number(size));
+    m_storeHSplitterSize = hSplitterSizes;
 
     for (const QString &path : asConst(m_addedScanDirs))
         ScanFoldersModel::instance()->removePath(path);
@@ -619,38 +630,18 @@ void OptionsDialog::changePage(QListWidgetItem *current, QListWidgetItem *previo
     m_ui->tabOption->setCurrentIndex(m_ui->tabSelection->row(current));
 }
 
-void OptionsDialog::loadWindowState()
-{
-    Utils::Gui::resize(this, Preferences::instance()->getPrefSize());
-}
-
 void OptionsDialog::loadSplitterState()
 {
-    const QStringList sizesStr = Preferences::instance()->getPrefHSplitterSizes();
-
     // width has been modified, use height as width reference instead
     const int width = Utils::Gui::scaledSize(this
         , (m_ui->tabSelection->item(TAB_UI)->sizeHint().height() * 2));
-    QList<int> sizes {width, (m_ui->hsplitter->width() - width)};
-    if (sizesStr.size() == 2)
-        sizes = {sizesStr.first().toInt(), sizesStr.last().toInt()};
-    m_ui->hsplitter->setSizes(sizes);
-}
+    const QStringList defaultSizes = {QString::number(width), QString::number(m_ui->hsplitter->width() - width)};
 
-void OptionsDialog::saveWindowState() const
-{
-    Preferences *const pref = Preferences::instance();
+    QList<int> splitterSizes;
+    for (const QString &string : asConst(m_storeHSplitterSize.get(defaultSizes)))
+        splitterSizes.append(string.toInt());
 
-    // window size
-    pref->setPrefSize(size());
-
-    // Splitter size
-    const QStringList sizesStr =
-    {
-        QString::number(m_ui->hsplitter->sizes().first()),
-        QString::number(m_ui->hsplitter->sizes().last())
-    };
-    pref->setPrefHSplitterSizes(sizesStr);
+    m_ui->hsplitter->setSizes(splitterSizes);
 }
 
 void OptionsDialog::saveOptions()
@@ -1612,6 +1603,8 @@ void OptionsDialog::setLocale(const QString &localeStr)
         QLocale locale(localeStr);
         if (locale.language() == QLocale::Uzbek)
             name = "uz@Latn";
+        else if (locale.language() == QLocale::Azerbaijani)
+            name = "az@latin";
         else
             name = locale.name();
     }
