@@ -28,16 +28,23 @@
 
 #include "programupdater.h"
 
+#if defined(Q_OS_WIN)
+#include <Windows.h>
+#include <versionhelpers.h>  // must follow after Windows.h
+#endif
+
 #include <QDebug>
 #include <QDesktopServices>
 #include <QRegularExpression>
 #include <QStringList>
-#include <QSysInfo>
 #include <QXmlStreamReader>
 
-#include "base/net/downloadhandler.h"
+#if defined(Q_OS_WIN)
+#include <QSysInfo>
+#endif
+
 #include "base/net/downloadmanager.h"
-#include "base/utils/fs.h"
+#include "base/version.h"
 
 namespace
 {
@@ -56,36 +63,44 @@ void ProgramUpdater::checkForUpdates()
 {
     // Don't change this User-Agent. In case our updater goes haywire,
     // the filehost can identify it and contact us.
-    Net::DownloadHandler *handler = Net::DownloadManager::instance()->download(
-                Net::DownloadRequest(RSS_URL).userAgent("qBittorrent/" QBT_VERSION_2 " ProgramUpdater (www.qbittorrent.org)"));
-    connect(handler, static_cast<void (Net::DownloadHandler::*)(const QString &, const QByteArray &)>(&Net::DownloadHandler::downloadFinished)
-            , this, &ProgramUpdater::rssDownloadFinished);
-    connect(handler, &Net::DownloadHandler::downloadFailed, this, &ProgramUpdater::rssDownloadFailed);
+    Net::DownloadManager::instance()->download(
+                Net::DownloadRequest(RSS_URL).userAgent("qBittorrent/" QBT_VERSION_2 " ProgramUpdater (www.qbittorrent.org)")
+                , this, &ProgramUpdater::rssDownloadFinished);
 }
 
-void ProgramUpdater::rssDownloadFinished(const QString &url, const QByteArray &data)
+void ProgramUpdater::rssDownloadFinished(const Net::DownloadResult &result)
 {
-    Q_UNUSED(url);
+
+    if (result.status != Net::DownloadStatus::Success)
+    {
+        qDebug() << "Downloading the new qBittorrent updates RSS failed:" << result.errorString;
+        emit updateCheckFinished(false, QString(), m_invokedByUser);
+        return;
+    }
+
     qDebug("Finished downloading the new qBittorrent updates RSS");
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
     const QString OS_TYPE {"Mac OS X"};
 #elif defined(Q_OS_WIN)
-    const QString OS_TYPE {((QSysInfo::windowsVersion() >= QSysInfo::WV_WINDOWS7)
+    const QString OS_TYPE
+    {(::IsWindows7OrGreater()
             && QSysInfo::currentCpuArchitecture().endsWith("64"))
         ? "Windows x64" : "Windows"};
 #endif
 
     QString version;
-    QXmlStreamReader xml(data);
+    QXmlStreamReader xml(result.data);
     bool inItem = false;
     QString updateLink;
     QString type;
 
-    while (!xml.atEnd()) {
+    while (!xml.atEnd())
+    {
         xml.readNext();
 
-        if (xml.isStartElement()) {
+        if (xml.isStartElement())
+        {
             if (xml.name() == "item")
                 inItem = true;
             else if (inItem && xml.name() == "link")
@@ -95,11 +110,15 @@ void ProgramUpdater::rssDownloadFinished(const QString &url, const QByteArray &d
             else if (inItem && xml.name() == "version")
                 version = getStringValue(xml);
         }
-        else if (xml.isEndElement()) {
-            if (inItem && xml.name() == "item") {
-                if (type.compare(OS_TYPE, Qt::CaseInsensitive) == 0) {
+        else if (xml.isEndElement())
+        {
+            if (inItem && xml.name() == "item")
+            {
+                if (type.compare(OS_TYPE, Qt::CaseInsensitive) == 0)
+                {
                     qDebug("The last update available is %s", qUtf8Printable(version));
-                    if (!version.isEmpty()) {
+                    if (!version.isEmpty())
+                    {
                         qDebug("Detected version is %s", qUtf8Printable(version));
                         if (isVersionMoreRecent(version))
                             m_updateUrl = updateLink;
@@ -118,14 +137,6 @@ void ProgramUpdater::rssDownloadFinished(const QString &url, const QByteArray &d
     emit updateCheckFinished(!m_updateUrl.isEmpty(), version, m_invokedByUser);
 }
 
-void ProgramUpdater::rssDownloadFailed(const QString &url, const QString &error)
-{
-    Q_UNUSED(url);
-
-    qDebug() << "Downloading the new qBittorrent updates RSS failed:" << error;
-    emit updateCheckFinished(false, QString(), m_invokedByUser);
-}
-
 void ProgramUpdater::updateProgram()
 {
     Q_ASSERT(!m_updateUrl.isEmpty());
@@ -136,12 +147,14 @@ void ProgramUpdater::updateProgram()
 bool ProgramUpdater::isVersionMoreRecent(const QString &remoteVersion) const
 {
     const QRegularExpressionMatch regVerMatch = QRegularExpression("([0-9.]+)").match(QBT_VERSION);
-    if (regVerMatch.hasMatch()) {
-        QString localVersion = regVerMatch.captured(1);
-        qDebug() << Q_FUNC_INFO << "local version:" << localVersion << "/" << QBT_VERSION;
-        QStringList remoteParts = remoteVersion.split('.');
-        QStringList localParts = localVersion.split('.');
-        for (int i = 0; i < qMin(remoteParts.size(), localParts.size()); ++i) {
+    if (regVerMatch.hasMatch())
+    {
+        const QString localVersion = regVerMatch.captured(1);
+        const QVector<QStringRef> remoteParts = remoteVersion.splitRef('.');
+        const QVector<QStringRef> localParts = localVersion.splitRef('.');
+
+        for (int i = 0; i < qMin(remoteParts.size(), localParts.size()); ++i)
+        {
             if (remoteParts[i].toInt() > localParts[i].toInt())
                 return true;
             if (remoteParts[i].toInt() < localParts[i].toInt())
@@ -166,6 +179,6 @@ namespace
         if (xml.isCharacters() && !xml.isWhitespace())
             return xml.text().toString();
 
-        return QString();
+        return {};
     }
 }
