@@ -31,10 +31,20 @@
 #include <cmath>
 
 #include <QDebug>
+#include <QVector>
+
+namespace
+{
+    QColor dlPieceColor(const QColor &pieceColor)
+    {
+        const QColor green {Qt::green};
+        return QColor::fromHsl(green.hslHue(), pieceColor.hslSaturation(), pieceColor.lightness());
+    }
+}
 
 DownloadedPiecesBar::DownloadedPiecesBar(QWidget *parent)
     : base {parent}
-    , m_dlPieceColor {0, 0xd0, 0}
+    , m_dlPieceColor {dlPieceColor(pieceColor())}
 {
 }
 
@@ -50,7 +60,8 @@ QVector<float> DownloadedPiecesBar::bitfieldToFloatVector(const QBitArray &vecin
     // image.x(0) = pieces.x(0.0 >= x < 1.7)
     // image.x(1) = pieces.x(1.7 >= x < 3.4)
 
-    for (int x = 0; x < reqSize; ++x) {
+    for (int x = 0; x < reqSize; ++x)
+    {
         // R - real
         const float fromR = x * ratio;
         const float toR = (x + 1) * ratio;
@@ -71,15 +82,18 @@ QVector<float> DownloadedPiecesBar::bitfieldToFloatVector(const QBitArray &vecin
         float value = 0;
 
         // case when calculated range is (15.2 >= x < 15.7)
-        if (x2 == toCMinusOne) {
+        if (x2 == toCMinusOne)
+        {
             if (vecin[x2])
                 value += ratio;
             ++x2;
         }
         // case when (15.2 >= x < 17.8)
-        else {
+        else
+        {
             // subcase (15.2 >= x < 16)
-            if (x2 != fromR) {
+            if (x2 != fromR)
+            {
                 if (vecin[x2])
                     value += 1.0 - (fromR - fromC);
                 ++x2;
@@ -91,7 +105,8 @@ QVector<float> DownloadedPiecesBar::bitfieldToFloatVector(const QBitArray &vecin
                     value += 1.0;
 
             // subcase (17 >= x < 17.8)
-            if (x2 == toCMinusOne) {
+            if (x2 == toCMinusOne)
+            {
                 if (vecin[x2])
                     value += 1.0 - (toC - toR);
                 ++x2;
@@ -114,13 +129,15 @@ bool DownloadedPiecesBar::updateImage(QImage &image)
 {
     //  qDebug() << "updateImage";
     QImage image2(width() - 2 * borderWidth, 1, QImage::Format_RGB888);
-    if (image2.isNull()) {
+    if (image2.isNull())
+    {
         qDebug() << "QImage image2() allocation failed, width():" << width();
         return false;
     }
 
-    if (m_pieces.isEmpty()) {
-        image2.fill(Qt::white);
+    if (m_pieces.isEmpty())
+    {
+        image2.fill(backgroundColor());
         image = image2;
         return true;
     }
@@ -129,10 +146,12 @@ bool DownloadedPiecesBar::updateImage(QImage &image)
     QVector<float> scaledPiecesDl = bitfieldToFloatVector(m_downloadedPieces, image2.width());
 
     // filling image
-    for (int x = 0; x < scaledPieces.size(); ++x) {
+    for (int x = 0; x < scaledPieces.size(); ++x)
+    {
         float piecesToValue = scaledPieces.at(x);
         float piecesToValueDl = scaledPiecesDl.at(x);
-        if (piecesToValueDl != 0) {
+        if (piecesToValueDl != 0)
+        {
             float fillRatio = piecesToValue + piecesToValueDl;
             float ratio = piecesToValueDl / fillRatio;
 
@@ -141,7 +160,8 @@ bool DownloadedPiecesBar::updateImage(QImage &image)
 
             image2.setPixel(x, 0, mixedColor);
         }
-        else {
+        else
+        {
             image2.setPixel(x, 0, pieceColors()[piecesToValue * 255]);
         }
     }
@@ -157,12 +177,6 @@ void DownloadedPiecesBar::setProgress(const QBitArray &pieces, const QBitArray &
     requestImageUpdate();
 }
 
-void DownloadedPiecesBar::setColors(const QColor &background, const QColor &border, const QColor &complete, const QColor &incomplete)
-{
-    m_dlPieceColor = incomplete;
-    base::setColors(background, border, complete);
-}
-
 void DownloadedPiecesBar::clear()
 {
     m_pieces.clear();
@@ -172,7 +186,11 @@ void DownloadedPiecesBar::clear()
 
 QString DownloadedPiecesBar::simpleToolTipText() const
 {
-    return tr("White: Missing pieces") + '\n'
-           + tr("Green: Partial pieces") + '\n'
-           + tr("Blue: Completed pieces") + '\n';
+    const QString borderColor = colorBoxBorderColor().name();
+    const QString rowHTML = QString::fromLatin1("<tr><td width=20 bgcolor='%1' style='border: 1px solid \"%2\";'></td><td>%3</td></tr>");
+    return QLatin1String("<table cellspacing=4>")
+           + rowHTML.arg(backgroundColor().name(), borderColor, tr("Missing pieces"))
+           + rowHTML.arg(m_dlPieceColor.name(), borderColor, tr("Partial pieces"))
+           + rowHTML.arg(pieceColor().name(), borderColor, tr("Completed pieces"))
+           + QLatin1String("</table>");
 }
