@@ -28,11 +28,11 @@
 
 #include "torrentcontentmodel.h"
 
-#include <QDir>
+#include <algorithm>
+
 #include <QFileIconProvider>
 #include <QFileInfo>
 #include <QIcon>
-#include <QMap>
 
 #if defined(Q_OS_WIN)
 #include <Windows.h>
@@ -43,32 +43,26 @@
 #include <QMimeType>
 #endif
 
-#if defined Q_OS_WIN || defined Q_OS_MAC
+#if defined Q_OS_WIN || defined Q_OS_MACOS
 #define QBT_PIXMAP_CACHE_FOR_FILE_ICONS
 #include <QPixmapCache>
 #endif
 
-#include "base/bittorrent/filepriority.h"
+#include "base/bittorrent/downloadpriority.h"
+#include "base/bittorrent/torrentinfo.h"
 #include "base/global.h"
-#include "base/utils/misc.h"
 #include "base/utils/fs.h"
-#include "guiiconprovider.h"
 #include "torrentcontentmodelfile.h"
 #include "torrentcontentmodelfolder.h"
 #include "torrentcontentmodelitem.h"
+#include "uithememanager.h"
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_MACOS
 #include "macutilities.h"
 #endif
 
 namespace
 {
-    QIcon getDirectoryIcon()
-    {
-        static QIcon cached = GuiIconProvider::instance()->getIcon("inode-directory");
-        return cached;
-    }
-
     class UnifiedFileIconProvider : public QFileIconProvider
     {
     public:
@@ -77,46 +71,30 @@ namespace
         QIcon icon(const QFileInfo &info) const override
         {
             Q_UNUSED(info);
-            static QIcon cached = GuiIconProvider::instance()->getIcon("text-plain");
+            static QIcon cached = UIThemeManager::instance()->getIcon("text-plain");
             return cached;
         }
     };
 
 #ifdef QBT_PIXMAP_CACHE_FOR_FILE_ICONS
-    struct Q_DECL_UNUSED PixmapCacheSetup
-    {
-        static const int PixmapCacheForIconsSize = 2 * 1024 * 1024; // 2 MiB for file icons
-
-        PixmapCacheSetup()
-        {
-            QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() + PixmapCacheForIconsSize);
-        }
-
-        ~PixmapCacheSetup()
-        {
-            Q_ASSERT(QPixmapCache::cacheLimit() > PixmapCacheForIconsSize);
-            QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() - PixmapCacheForIconsSize);
-        }
-    };
-
-    PixmapCacheSetup pixmapCacheSetup;
-
     class CachingFileIconProvider : public UnifiedFileIconProvider
     {
     public:
         using QFileIconProvider::icon;
 
-        QIcon icon(const QFileInfo &info) const final override
+        QIcon icon(const QFileInfo &info) const final
         {
             const QString ext = info.suffix();
-            if (!ext.isEmpty()) {
+            if (!ext.isEmpty())
+            {
                 QPixmap cached;
-                if (QPixmapCache::find(ext, &cached)) return QIcon(cached);
+                if (QPixmapCache::find(ext, &cached)) return {cached};
 
                 const QPixmap pixmap = pixmapForExtension(ext);
-                if (!pixmap.isNull()) {
+                if (!pixmap.isNull())
+                {
                     QPixmapCache::insert(ext, pixmap);
-                    return QIcon(pixmap);
+                    return {pixmap};
                 }
             }
             return UnifiedFileIconProvider::icon(info);
@@ -138,14 +116,14 @@ namespace
             HRESULT hr = ::SHGetFileInfoW(extWithDot.toStdWString().c_str(),
                 FILE_ATTRIBUTE_NORMAL, &sfi, sizeof(sfi), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES);
             if (FAILED(hr))
-                return QPixmap();
+                return {};
 
             QPixmap iconPixmap = QtWin::fromHICON(sfi.hIcon);
             ::DestroyIcon(sfi.hIcon);
             return iconPixmap;
         }
     };
-#elif defined(Q_OS_MAC)
+#elif defined(Q_OS_MACOS)
     // There is a similar bug on macOS, to be reported to Qt
     // https://github.com/qbittorrent/qBittorrent/pull/6156#issuecomment-316302615
     class MacFileIconProvider final : public CachingFileIconProvider
@@ -184,12 +162,14 @@ namespace
         {
             const QMimeType mimeType = m_db.mimeTypeForFile(info, QMimeDatabase::MatchExtension);
             QIcon res = QIcon::fromTheme(mimeType.iconName());
-            if (!res.isNull()) {
+            if (!res.isNull())
+            {
                 return res;
             }
 
             res = QIcon::fromTheme(mimeType.genericIconName());
-            if (!res.isNull()) {
+            if (!res.isNull())
+            {
                 return res;
             }
 
@@ -204,11 +184,11 @@ namespace
 
 TorrentContentModel::TorrentContentModel(QObject *parent)
     : QAbstractItemModel(parent)
-    , m_rootItem(new TorrentContentModelFolder(QList<QVariant>({ tr("Name"), tr("Size"), tr("Progress"), tr("Download Priority"), tr("Remaining"), tr("Availability") })))
+    , m_rootItem(new TorrentContentModelFolder(QVector<QString>({ tr("Name"), tr("Size"), tr("Progress"), tr("Download Priority"), tr("Remaining"), tr("Availability") })))
 {
 #if defined(Q_OS_WIN)
     m_fileIconProvider = new WinShellFileIconProvider();
-#elif defined(Q_OS_MAC)
+#elif defined(Q_OS_MACOS)
     m_fileIconProvider = new MacFileIconProvider();
 #else
     static bool doesBuiltInProviderWork = doesQFileIconProviderWork();
@@ -234,10 +214,10 @@ void TorrentContentModel::updateFilesProgress(const QVector<qreal> &fp)
     // Update folders progress in the tree
     m_rootItem->recalculateProgress();
     m_rootItem->recalculateAvailability();
-    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+    emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)));
 }
 
-void TorrentContentModel::updateFilesPriorities(const QVector<int> &fprio)
+void TorrentContentModel::updateFilesPriorities(const QVector<BitTorrent::DownloadPriority> &fprio)
 {
     Q_ASSERT(m_filesIndex.size() == fprio.size());
     // XXX: Why is this necessary?
@@ -246,8 +226,8 @@ void TorrentContentModel::updateFilesPriorities(const QVector<int> &fprio)
 
     emit layoutAboutToBeChanged();
     for (int i = 0; i < fprio.size(); ++i)
-        m_filesIndex[i]->setPriority(static_cast<BitTorrent::FilePriority>(fprio[i]));
-    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+        m_filesIndex[i]->setPriority(static_cast<BitTorrent::DownloadPriority>(fprio[i]));
+    emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)));
 }
 
 void TorrentContentModel::updateFilesAvailability(const QVector<qreal> &fa)
@@ -257,36 +237,36 @@ void TorrentContentModel::updateFilesAvailability(const QVector<qreal> &fa)
     if (m_filesIndex.size() != fa.size()) return;
 
     emit layoutAboutToBeChanged();
-    for (int i = 0; i < fa.size(); ++i)
+    for (int i = 0; i < m_filesIndex.size(); ++i)
         m_filesIndex[i]->setAvailability(fa[i]);
     // Update folders progress in the tree
     m_rootItem->recalculateProgress();
-    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+    emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)));
 }
 
-QVector<int> TorrentContentModel::getFilePriorities() const
+QVector<BitTorrent::DownloadPriority> TorrentContentModel::getFilePriorities() const
 {
-    QVector<int> prio;
+    QVector<BitTorrent::DownloadPriority> prio;
     prio.reserve(m_filesIndex.size());
     for (const TorrentContentModelFile *file : asConst(m_filesIndex))
-        prio.push_back(static_cast<int>(file->priority()));
+        prio.push_back(file->priority());
     return prio;
 }
 
 bool TorrentContentModel::allFiltered() const
 {
-    for (const TorrentContentModelFile *fileItem : asConst(m_filesIndex))
-        if (fileItem->priority() != BitTorrent::FilePriority::Ignored)
-            return false;
-    return true;
+    return std::all_of(m_filesIndex.cbegin(), m_filesIndex.cend(), [](const TorrentContentModelFile *fileItem)
+    {
+        return (fileItem->priority() == BitTorrent::DownloadPriority::Ignored);
+    });
 }
 
 int TorrentContentModel::columnCount(const QModelIndex &parent) const
 {
     if (parent.isValid())
         return static_cast<TorrentContentModelItem*>(parent.internalPointer())->columnCount();
-    else
-        return m_rootItem->columnCount();
+
+    return m_rootItem->columnCount();
 }
 
 bool TorrentContentModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -294,35 +274,39 @@ bool TorrentContentModel::setData(const QModelIndex &index, const QVariant &valu
     if (!index.isValid())
         return false;
 
-    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::CheckStateRole)) {
-        TorrentContentModelItem *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
+    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::CheckStateRole))
+    {
+        auto *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
         qDebug("setData(%s, %d", qUtf8Printable(item->name()), value.toInt());
-        if (static_cast<int>(item->priority()) != value.toInt()) {
-            BitTorrent::FilePriority prio = BitTorrent::FilePriority::Normal;
+        if (static_cast<int>(item->priority()) != value.toInt())
+        {
+            BitTorrent::DownloadPriority prio = BitTorrent::DownloadPriority::Normal;
             if (value.toInt() == Qt::PartiallyChecked)
-                prio = BitTorrent::FilePriority::Mixed;
+                prio = BitTorrent::DownloadPriority::Mixed;
             else if (value.toInt() == Qt::Unchecked)
-                prio = BitTorrent::FilePriority::Ignored;
+                prio = BitTorrent::DownloadPriority::Ignored;
 
             item->setPriority(prio);
             // Update folders progress in the tree
             m_rootItem->recalculateProgress();
             m_rootItem->recalculateAvailability();
-            emit dataChanged(this->index(0, 0), this->index(rowCount() - 1, columnCount() - 1));
+            emit dataChanged(this->index(0, 0), this->index((rowCount() - 1), (columnCount() - 1)));
             emit filteredFilesChanged();
         }
         return true;
     }
 
-    if (role == Qt::EditRole) {
+    if (role == Qt::EditRole)
+    {
         Q_ASSERT(index.isValid());
-        TorrentContentModelItem *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
-        switch (index.column()) {
+        auto *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
+        switch (index.column())
+        {
         case TorrentContentModelItem::COL_NAME:
             item->setName(value.toString());
             break;
         case TorrentContentModelItem::COL_PRIO:
-            item->setPriority(static_cast<BitTorrent::FilePriority>(value.toInt()));
+            item->setPriority(static_cast<BitTorrent::DownloadPriority>(value.toInt()));
             break;
         default:
             return false;
@@ -341,7 +325,7 @@ TorrentContentModelItem::ItemType TorrentContentModel::itemType(const QModelInde
 
 int TorrentContentModel::getFileIndex(const QModelIndex &index)
 {
-    TorrentContentModelItem *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
+    auto *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
     if (item->itemType() == TorrentContentModelItem::FileType)
         return static_cast<TorrentContentModelFile*>(item)->fileIndex();
 
@@ -352,29 +336,47 @@ int TorrentContentModel::getFileIndex(const QModelIndex &index)
 QVariant TorrentContentModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid())
-        return QVariant();
+        return {};
 
-    TorrentContentModelItem *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
+    auto *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
 
-    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::DecorationRole)) {
-        if (item->itemType() == TorrentContentModelItem::FolderType)
-            return getDirectoryIcon();
-        else
+    switch (role)
+    {
+    case Qt::DecorationRole:
+    {
+            if (index.column() != TorrentContentModelItem::COL_NAME)
+                return {};
+
+            if (item->itemType() == TorrentContentModelItem::FolderType)
+                return m_fileIconProvider->icon(QFileIconProvider::Folder);
             return m_fileIconProvider->icon(QFileInfo(item->name()));
+        }
+    case Qt::CheckStateRole:
+    {
+            if (index.column() != TorrentContentModelItem::COL_NAME)
+                return {};
+
+            if (item->priority() == BitTorrent::DownloadPriority::Ignored)
+                return Qt::Unchecked;
+            if (item->priority() == BitTorrent::DownloadPriority::Mixed)
+                return Qt::PartiallyChecked;
+            return Qt::Checked;
+        }
+    case Qt::TextAlignmentRole:
+        if ((index.column() == TorrentContentModelItem::COL_SIZE)
+            || (index.column() == TorrentContentModelItem::COL_REMAINING))
+            return QVariant {Qt::AlignRight | Qt::AlignVCenter};
+        return {};
+
+    case Qt::DisplayRole:
+        return item->displayData(index.column());
+
+    case Roles::UnderlyingDataRole:
+        return item->underlyingData(index.column());
+
+    default:
+        return {};
     }
-
-    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::CheckStateRole)) {
-        if (item->data(TorrentContentModelItem::COL_PRIO).toInt() == static_cast<int>(BitTorrent::FilePriority::Ignored))
-            return Qt::Unchecked;
-        if (item->data(TorrentContentModelItem::COL_PRIO).toInt() == static_cast<int>(BitTorrent::FilePriority::Mixed))
-            return Qt::PartiallyChecked;
-        return Qt::Checked;
-    }
-
-    if (role == Qt::DisplayRole)
-        return item->data(index.column());
-
-    return QVariant();
 }
 
 Qt::ItemFlags TorrentContentModel::flags(const QModelIndex &index) const
@@ -382,27 +384,43 @@ Qt::ItemFlags TorrentContentModel::flags(const QModelIndex &index) const
     if (!index.isValid())
         return Qt::NoItemFlags;
 
+    Qt::ItemFlags flags {Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable};
     if (itemType(index) == TorrentContentModelItem::FolderType)
-        return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsTristate;
+        flags |= Qt::ItemIsTristate;
+    if (index.column() == TorrentContentModelItem::COL_PRIO)
+        flags |= Qt::ItemIsEditable;
 
-    return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsUserCheckable;
+    return flags;
 }
 
 QVariant TorrentContentModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    if ((orientation == Qt::Horizontal) && (role == Qt::DisplayRole))
-        return m_rootItem->data(section);
+    if (orientation != Qt::Horizontal)
+        return {};
 
-    return QVariant();
+    switch (role)
+    {
+    case Qt::DisplayRole:
+        return m_rootItem->displayData(section);
+
+    case Qt::TextAlignmentRole:
+        if ((section == TorrentContentModelItem::COL_SIZE)
+            || (section == TorrentContentModelItem::COL_REMAINING))
+            return QVariant {Qt::AlignRight | Qt::AlignVCenter};
+        return {};
+
+    default:
+        return {};
+    }
 }
 
 QModelIndex TorrentContentModel::index(int row, int column, const QModelIndex &parent) const
 {
     if (parent.isValid() && (parent.column() != 0))
-        return QModelIndex();
+        return {};
 
     if (column >= TorrentContentModelItem::NB_COL)
-        return QModelIndex();
+        return {};
 
     TorrentContentModelFolder *parentItem;
     if (!parent.isValid())
@@ -412,26 +430,26 @@ QModelIndex TorrentContentModel::index(int row, int column, const QModelIndex &p
     Q_ASSERT(parentItem);
 
     if (row >= parentItem->childCount())
-        return QModelIndex();
+        return {};
 
     TorrentContentModelItem *childItem = parentItem->child(row);
     if (childItem)
         return createIndex(row, column, childItem);
-    return QModelIndex();
+    return {};
 }
 
 QModelIndex TorrentContentModel::parent(const QModelIndex &index) const
 {
     if (!index.isValid())
-        return QModelIndex();
+        return {};
 
-    TorrentContentModelItem *childItem = static_cast<TorrentContentModelItem*>(index.internalPointer());
+    auto *childItem = static_cast<TorrentContentModelItem*>(index.internalPointer());
     if (!childItem)
-        return QModelIndex();
+        return {};
 
     TorrentContentModelItem *parentItem = childItem->parent();
     if (parentItem == m_rootItem)
-        return QModelIndex();
+        return {};
 
     return createIndex(parentItem->row(), 0, parentItem);
 }
@@ -473,17 +491,21 @@ void TorrentContentModel::setupModelData(const BitTorrent::TorrentInfo &info)
 
     TorrentContentModelFolder *currentParent;
     // Iterate over files
-    for (int i = 0; i < filesCount; ++i) {
+    for (int i = 0; i < filesCount; ++i)
+    {
         currentParent = m_rootItem;
-        QString path = Utils::Fs::fromNativePath(info.filePath(i));
+        const QString path = Utils::Fs::toUniformPath(info.filePath(i));
+
         // Iterate of parts of the path to create necessary folders
-        QStringList pathFolders = path.split('/', QString::SkipEmptyParts);
+        QVector<QStringRef> pathFolders = path.splitRef('/', QString::SkipEmptyParts);
         pathFolders.removeLast();
-        for (const QString &pathPart : asConst(pathFolders)) {
-            if (pathPart == ".unwanted")
-                continue;
-            TorrentContentModelFolder* newParent = currentParent->childFolderWithName(pathPart);
-            if (!newParent) {
+
+        for (const QStringRef &pathPartRef : asConst(pathFolders))
+        {
+            const QString pathPart = pathPartRef.toString();
+            TorrentContentModelFolder *newParent = currentParent->childFolderWithName(pathPart);
+            if (!newParent)
+            {
                 newParent = new TorrentContentModelFolder(pathPart, currentParent);
                 currentParent->appendChild(newParent);
             }
@@ -499,17 +521,18 @@ void TorrentContentModel::setupModelData(const BitTorrent::TorrentInfo &info)
 
 void TorrentContentModel::selectAll()
 {
-    for (int i = 0; i < m_rootItem->childCount(); ++i) {
+    for (int i = 0; i < m_rootItem->childCount(); ++i)
+    {
         TorrentContentModelItem* child = m_rootItem->child(i);
-        if (child->priority() == BitTorrent::FilePriority::Ignored)
-            child->setPriority(BitTorrent::FilePriority::Normal);
+        if (child->priority() == BitTorrent::DownloadPriority::Ignored)
+            child->setPriority(BitTorrent::DownloadPriority::Normal);
     }
-    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+    emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)));
 }
 
 void TorrentContentModel::selectNone()
 {
     for (int i = 0; i < m_rootItem->childCount(); ++i)
-        m_rootItem->child(i)->setPriority(BitTorrent::FilePriority::Ignored);
-    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+        m_rootItem->child(i)->setPriority(BitTorrent::DownloadPriority::Ignored);
+    emit dataChanged(index(0, 0), index((rowCount() - 1), (columnCount() - 1)));
 }
