@@ -31,6 +31,7 @@
 #include <algorithm>
 
 #include <QHash>
+#include <QVector>
 
 #include "base/bittorrent/trackerentry.h"
 #include "ui_trackerentriesdialog.h"
@@ -41,7 +42,7 @@
 TrackerEntriesDialog::TrackerEntriesDialog(QWidget *parent)
     : QDialog(parent)
     , m_ui(new Ui::TrackerEntriesDialog)
-    , m_storeDialogSize(SETTINGS_KEY("Dimension"))
+    , m_storeDialogSize(SETTINGS_KEY("Size"))
 {
     m_ui->setupUi(this);
 
@@ -63,7 +64,8 @@ void TrackerEntriesDialog::setTrackers(const QVector<BitTorrent::TrackerEntry> &
     int maxTier = -1;
     QHash<int, QString> tiers;  // <tier, tracker URLs>
 
-    for (const BitTorrent::TrackerEntry &entry : trackers) {
+    for (const BitTorrent::TrackerEntry &entry : trackers)
+    {
         tiers[entry.tier()] += (entry.url() + '\n');
         maxTier = std::max(maxTier, entry.tier());
     }
@@ -85,10 +87,12 @@ QVector<BitTorrent::TrackerEntry> TrackerEntriesDialog::trackers() const
     entries.reserve(lines.size());
 
     int tier = 0;
-    for (QStringRef line : lines) {
+    for (QStringRef line : lines)
+    {
         line = line.trimmed();
 
-        if (line.isEmpty()) {
+        if (line.isEmpty())
+        {
             ++tier;
             continue;
         }
