@@ -32,7 +32,7 @@
 
 namespace BitTorrent
 {
-    class TorrentHandle;
+    class Torrent;
 }
 
 // Torrent keys
@@ -58,9 +58,11 @@ const char KEY_TORRENT_TAGS[] = "tags";
 const char KEY_TORRENT_SUPER_SEEDING[] = "super_seeding";
 const char KEY_TORRENT_FORCE_START[] = "force_start";
 const char KEY_TORRENT_SAVE_PATH[] = "save_path";
+const char KEY_TORRENT_CONTENT_PATH[] = "content_path";
 const char KEY_TORRENT_ADDED_ON[] = "added_on";
 const char KEY_TORRENT_COMPLETION_ON[] = "completion_on";
 const char KEY_TORRENT_TRACKER[] = "tracker";
+const char KEY_TORRENT_TRACKERS_COUNT[] = "trackers_count";
 const char KEY_TORRENT_DL_LIMIT[] = "dl_limit";
 const char KEY_TORRENT_UP_LIMIT[] = "up_limit";
 const char KEY_TORRENT_AMOUNT_DOWNLOADED[] = "downloaded";
@@ -80,4 +82,4 @@ const char KEY_TORRENT_AUTO_TORRENT_MANAGEMENT[] = "auto_tmm";
 const char KEY_TORRENT_TIME_ACTIVE[] = "time_active";
 const char KEY_TORRENT_AVAILABILITY[] = "availability";
 
-QVariantMap serialize(const BitTorrent::TorrentHandle &torrent);
+QVariantMap serialize(const BitTorrent::Torrent &torrent);
